from shapely.geometry import Point

__all__ = ['point_to_xy', 'linestring_first_point', 'linestring_last_point']


def point_to_xy(row) -> tuple:
    """
    A function that can be applied to a GeoDataFrame to create columns of the x and y locations of a Point.
    All rows must contain Point geometry.

    Examples:
        # gdf: GeoDataFrame
        gdf[['x', 'y']] = gdf.apply(get_point_xy, axis=1)

    Args:
        row: the row of a GeoDataFrame from GeoPandas

    Returns:
        tuple (x, y)
    """
    return row.x, row.y


def linestring_first_point(row) -> Point:
    """
    A function that can be applied to a GeoDataFrame to extract the location of the first point of a LineString.
    All rows must contain LineString data.

    Examples:
        # gdf: GeoDataFrame
        gdf['line_first_point'] = gdf.apply(linestring_first_point, axis=1)

    Args:
        row: the row of a GeoDataFrame from GeoPandas

    Returns:
        shapely.geometry.Point
    """
    return Point(row.geometry.coords[0])


def linestring_last_point(row) -> Point:
    """
    A function that can be applied to a GeoDataFrame to extract the location of the last point of a LineString.
    All rows must contain LineString data.

    Examples:
        # gdf: GeoDataFrame
        gdf['line_last_point'] = gdf.apply(linestring_last_point, axis=1)

    Args:
        row: the row of a GeoDataFrame from GeoPandas

    Returns:
        shapely.geometry.Point
    """
    return Point(row.geometry.coords[-1])



import datetime
from dateutil import relativedelta as durt

import pandas as pd

__all__ = ['time_offset', 'window_avg']


def time_offset(df: pd.DataFrame, label: str = '_timeoffset',
                years: int = 0, months: int = 0,
                days: int = 0, hours: int = 0, minutes: int = 0, seconds: int = 0) -> pd.DataFrame:
    """
    Make a duplicate dataframe where the datetime offset has been offset by given amount

    Recommended usage- make a time offset dataframe and then merge it with the original dataset

    Args:
        df: the pd.DataFrame to compute time offsets for
        label: a suffix to be applied to each column's name
        years: years to offset used by dateutil.relativedelta
        months: months to offset used by dateutil.relativedelta
        days: days to offset used by datetime.timedelta
        hours: hours to offset used by datetime.timedelta
        minutes: minutes to offset used by datetime.timedelta
        seconds: seconds to offset used by datetime.timedelta

    Returns:
        pd.DataFrame
    """
    df1 = df.copy()

    du_offset = durt.relativedelta(years=years, months=months)
    dt_offset = datetime.timedelta(days=days, hours=hours, minutes=minutes, seconds=seconds)

    df1.index = [i + du_offset + dt_offset for i in df1.index]
    df1.columns = [f'{i}{label}' for i in df1.columns]
    return df1


def window_avg(df: pd.DataFrame, steps: int = 1, label: str = '_winavg',) -> pd.DataFrame:
    """
    Make a duplicate dataframe where the rows are rolling window averages.

    Args:
        df: the pd.DataFrame to compute window averages for
        steps: the number of steps (rows) to average including the row at the end of the window
        label: a suffix to be applied to each column's name

    Returns:
        pd.DataFrame
    """
    df1 = df.copy()
    df1 = df1.rolling(window=steps).mean()
    df1.dropna(inplace=True)
    df1.columns = [f'{i}{label}' for i in df1.columns]
    return df1



import math

import numpy as np
import pandas as pd

__all__ = ['gumbel1', 'fdc', 'downstream', 'upstream', ]


def gumbel1(std: float, xbar: float, rp: int or float) -> float:
    """
    Solves the Gumbel Type I probability distribution function (pdf) = exp(-exp(-b)) where b is the covariate. Provide
    the standard deviation and mean of the list of annual maximum flows. Compare scipy.stats.gumbel_r

    Args:
        std (float): the standard deviation of the series
        xbar (float): the mean of the series
        rp (int or float): the return period in years

    Returns:
        float, the flow corresponding to the return period specified
    """
    # xbar = statistics.mean(year_max_flow_list)
    # std = statistics.stdev(year_max_flow_list, xbar=xbar)
    return -math.log(-math.log(1 - (1 / rp))) * std * .7797 + xbar - (.45 * std)


def fdc(a: list or np.array, steps: int = 500, exceed: bool = True) -> pd.DataFrame:
    """
    Generate the flow duration curve for a provided series of data

    Args:
        a (list or np.array): the flow values for which to compute the flow duration curve
        steps (int): the number of intervals between 0 and 100 for which to compute the exceedance probabilities
        exceed (bool): whether to show the exceedance probabilities (True) or non-exceedance probabilities (False)

    Returns:
        pd.DataFrame with 2 columns, probability and flow
    """
    percentiles = [round((100 / steps) * i, 5) for i in range(steps + 1)]
    flows = np.nanpercentile(a, percentiles)
    columns = ['probability', 'flow']
    if exceed:
        percentiles.reverse()
    return pd.DataFrame(np.transpose([percentiles, flows]), columns=columns)


def downstream(df: pd.DataFrame, target_id: int, id_col: str, next_col: str, order_col: str = None,
               same_order: bool = False, outlet_next_id: str or int = -1) -> tuple:
    """
    Traverse a stream network table containing a column of unique ID's, a column of the ID for the stream/basin
    downstream of that point, and, optionally, a column containing the stream order.

    Args:
        df (pd.DataFrame): a pandas DataFrame containing the id_col, next_col, and order_col if same_order is True
        target_id (int): the ID of the stream to begin the search from
        id_col (str): name of the DataFrame column which contains stream/basin ID's
        next_col (str): name of the DataFrame column which contains stream/basin ID's of the downstream segments
        order_col (str): name of the DataFrame column which contains stream orders
        same_order (bool): True limits searching to streams of the same order as the starting stream. False searches
            all streams until the outlet is found
        outlet_next_id (str or int): The placeholder value used as the downstream stream ID value at the outlet

    Returns:
        Tuple of stream ids in the order they come from the starting point.
    """
    downstream_ids = [target_id, ]

    df_ = df.copy()
    if same_order:
        df_ = df_[df_[order_col] == df_[df_[id_col] == target_id][order_col].values[0]]

    stream_row = df_[df_[id_col] == target_id]
    while stream_row[next_col].values[0] != outlet_next_id:
        downstream_ids.append(stream_row[next_col].values[0])
        stream_row = df_[df_[id_col] == stream_row[next_col].values[0]]
        if len(stream_row) == 0:
            break
    return tuple(downstream_ids)


def upstream(df: pd.DataFrame, target_id: int, id_col: str, next_col: str, order_col: str = None,
             same_order: bool = False) -> tuple:
    """
    Traverse a stream network table containing a column of unique ID's, a column of the ID for the stream/basin
    downstream of that point, and, optionally, a column containing the stream order.

    Args:
        df (pd.DataFrame): a pandas DataFrame containing the id_col, next_col, and order_col if same_order is True
        target_id (int): the ID of the stream to begin the search from
        id_col (str): name of the DataFrame column which contains stream/basin ID's
        next_col (str): name of the DataFrame column which contains stream/basin ID's of the downstream segments
        order_col (str): name of the DataFrame column which contains stream orders
        same_order (bool): True limits searching to streams of the same order as the starting stream. False searches
            all streams until the head of each branch is found

    Returns:
        Tuple of stream ids in the order they come from the starting point. If you chose same_order = False, the
        streams will appear in order on each upstream branch but the various branches will appear mixed in the tuple in
        the order they were encountered by the iterations.
    """
    df_ = df.copy()
    if same_order:
        df_ = df_[df_[order_col] == df_[df_[id_col] == target_id][order_col].values[0]]

    # start a list of the upstream ids
    upstream_ids = [target_id, ]
    upstream_rows = df_[df_[next_col] == target_id]

    while not upstream_rows.empty or len(upstream_rows) > 0:
        if len(upstream_rows) == 1:
            upstream_ids.append(upstream_rows[id_col].values[0])
            upstream_rows = df_[df_[next_col] == upstream_rows[id_col].values[0]]
        elif len(upstream_rows) > 1:
            for s_id in upstream_rows[id_col].values.tolist():
                upstream_ids += list(upstream(df_, s_id, id_col, next_col, order_col, same_order))
                upstream_rows = df_[df_[next_col] == s_id]
    return tuple(set(upstream_ids))



import numpy as np
import pandas as pd


class Sudoku:
    def __init__(self, grid: str or np.ndarray, **kwargs):
        if isinstance(grid, str):
            self.grid = pd.read_csv(grid, names=list(range(1, 10))).replace(kwargs.get('blank', 0), np.nan).values
        elif isinstance(grid, np.ndarray):
            self.grid = grid
        else:
            raise TypeError('The grid argument should be a str path to a csv file or a np 9x9 array')

        self.solution = self.grid
        self.is_solved = False

        # solver configs
        self.steps = ''
        self.iter = 0
        self.max_iter = kwargs.get('max_iter', 50)

    def solve(self):
        assigned = True
        self.iter = 0
        while assigned:
            made_progress = []
            for n in range(1, 10):
                options = self._annotate(n)
                made_progress.append(self._assign_rows(options, n))
                made_progress.append(self._assign_rows(options, n))
                made_progress.append(self._assign_boxes(options, n))
            assigned = any(made_progress)
            if np.nansum(self.solution) == 9 * 45:
                print('solved')
                self.is_solved = True
                return self.solution
            elif self.iter > self.max_iter:
                print('exceeded 50 iterations')
                return
            self.iter += 1

        print('out of logical solution steps')
        return self.solution

    def _annotate(self, n: int):
        # all empty squares are options, if they have a number in them then they are not
        options = np.where(np.isnan(self.solution), True, False)

        number_mask = np.where(self.solution == n, True, False)
        # annotate rows based on known locations
        for idx, row in enumerate(np.rollaxis(number_mask, 0)):
            if any(row):
                options[idx, :] = False

        # annotate columns based on known locations
        for idx, col in enumerate(np.rollaxis(number_mask, 1)):
            if any(col):
                options[:, idx] = False

        # annotate boxes based on known locations (if the num is in the box, other box cells aren't options)
        for _3row_idx in range(3):
            for _3col_idx in range(3):
                row_slice = slice(3 * _3row_idx, 3 * (_3row_idx + 1))
                col_slice = slice(3 * _3col_idx, 3 * (_3col_idx + 1))

                # if the number is in the box, none of the other cells need to be considered options
                if np.any(self.solution[row_slice, col_slice] == n):
                    options[row_slice, col_slice] = False

                # extrapolate remaining possibilities
                box = options[row_slice, col_slice].reshape(3, 3)
                num_true = sum(box.flatten())

                # the following extrapolations only apply if there are 2 or 3 options in the box
                if num_true not in (2, 3):
                    continue

                # if all the options are in the same row/col of the box then number will go in that row/col of the box
                # so we can rule out other open spaces in the row of the sudoku grid
                for idx, row in enumerate(np.rollaxis(box, 0)):
                    if sum(row) == num_true:
                        columns_to_rule_out = list(range(9))
                        columns_to_rule_out.remove(3 * _3col_idx)
                        columns_to_rule_out.remove(3 * _3col_idx + 1)
                        columns_to_rule_out.remove(3 * _3col_idx + 2)
                        for c in columns_to_rule_out:
                            options[3 * _3row_idx + idx, c] = False
                for idx, col in enumerate(np.rollaxis(box, 1)):
                    if sum(col) == num_true:
                        rows_to_rule_out = list(range(9))
                        rows_to_rule_out.remove(3 * _3row_idx)
                        rows_to_rule_out.remove(3 * _3row_idx + 1)
                        rows_to_rule_out.remove(3 * _3row_idx + 2)
                        for r in rows_to_rule_out:
                            options[r, 3 * _3col_idx + idx] = False

        return options

    def _assign_rows(self, options: np.array, n: int):
        # assign rows: if there is only 1 true in the row, it must go there
        made_assignment = False
        for idx, row in enumerate(np.rollaxis(options, 0)):
            unq_vals, unq_idx, unq_count = np.unique(row, return_index=True, return_counts=True)
            if True in unq_vals and unq_count[-1] == 1:
                self.solution[idx, unq_idx[-1]] = n
                self.steps += f'assigned {n} at grid cell ({idx}, {unq_idx[-1]}) -> row unique\n'
                made_assignment = True
        return made_assignment

    def _assign_cols(self, options: np.array, n: int):
        # assign cols: if there is only 1 true in the column, it must go there
        made_assignment = False
        for idx, col in enumerate(np.rollaxis(options, 1)):
            unq_vals, unq_idx, unq_count = np.unique(col, return_index=True, return_counts=True)
            if True in unq_vals and unq_count[-1] == 1:
                self.solution[unq_idx[-1], idx] = n
                self.steps += f'assigned {n} at grid cell ({unq_idx[-1]}, {idx}) -> col unique\n'
                made_assignment = True
        return made_assignment

    def _assign_boxes(self, options: np.array, n: int):
        # assign boxes: if there is only 1 true in the box, it must go there
        made_assignment = False
        for _3row_idx in range(3):
            for _3col_idx in range(3):
                row_slice = slice(3 * _3row_idx, 3 * (_3row_idx + 1))
                col_slice = slice(3 * _3col_idx, 3 * (_3col_idx + 1))
                unq_vals, unq_idx, unq_count = np.unique(
                    options[row_slice, col_slice], return_index=True, return_counts=True)
                if True in unq_vals and unq_count[-1] == 1:
                    if unq_idx[-1] >= 6:
                        row = 3 * _3row_idx + 2
                        col = 3 * _3col_idx + unq_idx[-1] - 6
                    elif unq_idx[-1] >= 3:
                        row = 3 * _3row_idx + 1
                        col = 3 * _3col_idx + unq_idx[-1] - 3
                    else:
                        row = 3 * _3row_idx
                        col = 3 * _3col_idx + unq_idx[-1]
                    self.steps += f'assigned {n} at grid cell ({row}, {col}) -> box unique\n'
                    self.solution[row, col] = n
                    made_assignment = True
        return made_assignment

    def to_csv(self, path: str) -> None:
        pd.DataFrame(self.solution).astype(int).to_csv(path, index=False, header=False)

    def to_html(self) -> str:
        return pd.DataFrame(self.solution).astype(int).to_html(index=False, header=False)



import rch.arrays
import rch.df
import rch.gdf
import rch.hydro
import rch.sudoku
import rch.web

__all__ = ['arrays', 'df', 'gdf', 'hydro', 'sudoku', 'web']
__version__ = '0.13'
__author__ = 'Riley Hales'
__license__ = 'BSD Clear'



import numpy as np
import pandas as pd

__all__ = ['idw', 'resample_1d', 'resample_2d', 'checkerboard']


def idw(a: np.array, loc: tuple, p: int = 1, r: int or float = None, nearest: int = None, bound: int = None) -> float:
    """
    Computes the inverse distance weighted interpolated value at a specified location (loc) from an array of measured
    values (a). There are 3 ways to limit the interpolation points considered.

    1. Use a search radius: only points within a certain radius of the location to be interpolated will be considered
    2. Choose a number of nearest neighbors to use.
    3. Use a number of points on each side that bound the interpolation location.

    All 3 may be applied but the most restrictive option will govern. For example, suppose you choose a radius of 10
    and to use the 5 nearest neighbors. If there are 100 points within the radius of 10, 95 of them will be ignored
    since you limited the interpolation to the nearest 5 points. Similarly, If you choose to take 5 points from each of
    the bounding quadrants and also the nearest 5 neighbors, then 15 of the points chosen because they bound the
    location will get ignored. This behavior could be intentional so all 3 variables can still be specified and applied.

    Args:
        a (np.array): a numpy array with 3 columns (x, y, value) and a row for each measurement
        loc (tuple): a tuple of (x, y) coordinates representing the location to get the interpolated values at
        p (int): an integer representing the power factor applied to the distances before inverting (usually 1, 2, 3)
        r (int or float): the radius, same length units as x & y values, to limit the value pairs in a. only points <=
            r distance away are used for the interpolation
        nearest (int): number of nearest points to include in the interpolation, if that many are available.
        bound (int): number of nearest points to include from the 4 bounding quadrants, if that many are available.

    Returns:
        float, the IDW interpolated value for the loc specified
    """
    # identify the x distance from location to the measurement points
    x = np.subtract(a[:, 0], loc[0])
    # identify the y distance from location to the measurement points
    y = np.subtract(a[:, 1], loc[1])
    # select the values column of the data
    val = a[:, 2]
    # compute the pythagorean distance (square root sum of the squares)
    dist = np.sqrt(np.add(np.multiply(x, x), np.multiply(y, y)))
    # filter the nearest number of points, limit by radius, and/or use bounding points (all use df sorted by distance)
    if r is not None or nearest is not None or bound is not None:
        b = pd.DataFrame({'dist': dist, 'val': val, 'x': x, 'y': y})
        b.sort_values('dist', inplace=True)
        if bound is not None:
            b = pd.concat((b.loc[(b['x'] >= 0) & (b['y'] > 0)].head(bound),
                           b.loc[(b['x'] >= 0) & (b['y'] < 0)].head(bound),
                           b.loc[(b['x'] < 0) & (b['y'] > 0)].head(bound),
                           b.loc[(b['x'] < 0) & (b['y'] < 0)].head(bound),))
        if nearest is not None:
            b = b.head(nearest)
        if r is not None:
            b = b[b['dist'] <= r]
        dist = b['dist'].values
        val = b['val'].values

    # raise distances to power (usually 1 or 2)
    dist = np.power(dist, p)
    # inverse the distances
    dist = np.divide(1, dist)

    return float(np.divide(np.sum(np.multiply(dist, val)), np.sum(dist)))


def random_xyv(n: int = 41, random: bool = False) -> pd.DataFrame:
    """
    Generates a symmetrical, square shaped grid centered at 0, 0 where the length of one side is n. N should be an odd
    number so that the grid can be symmetrical and include the x and y axis lines. This function will add 1 to the
    shape if you provide an even number.

    Args:
        n (int): an odd integer defining the length of the square grid's edges
        random (bool): True -> randomly generated values, False -> values = x * y

    Returns:
        pd.DateFrame with an 'x', 'y', and 'v' (value) column.
    """
    size = int(n)
    if size % 2 == 0:
        size += 1

    x = np.asarray([[i] * size for i in range(-(size // 2), (size // 2) + 1)]).flatten()
    y = np.asarray(list(range(-(size // 2), (size // 2) + 1)) * size).flatten()
    if random:
        return pd.DataFrame({'x': x, 'y': y, 'v': np.random.randint(-(size // 2), size // 2, size=(size * size,))})
    else:
        return pd.DataFrame({'x': x, 'y': y, 'v': x * y})


def resample_1d(a: np.array, f: int, stat: str = 'mean') -> np.array:
    """
    Resamples the values in a 1D array by an integer factor using a specified stat type

    Args:
        a (np.array): A 1D array of numbers
        f (int): An integer resampling factor/ratio evenly divisible into len(a)
        stat (str): mean, max, min, median -> the method for aggregating the numeric values

    Returns:
        resampled np.array, ndim == 1,  shape == (a.shape[0] / f)
    """
    assert a.ndim == 1, "valid for 1D arrays only"
    assert a.shape[0] % f == 0, "length of axis=0 not evenly divisible by f0"
    assert stat in ('mean', 'max', 'min', 'median'), "choose either mean, max, min or median for aggregation stat"

    if stat == 'mean':
        return np.array([np.nanmean(i) for i in np.split(a, a.shape[0] / f)])
    elif stat == 'max':
        return np.array([np.nanmax(i) for i in np.split(a, a.shape[0] / f)])
    elif stat == 'min':
        return np.array([np.nanmin(i) for i in np.split(a, a.shape[0] / f)])
    elif stat == 'median':
        return np.array([np.nanmedian(i) for i in np.split(a, a.shape[0] / f)])


def resample_2d(a: np.array, f0: int, f1: int, stat: str = 'mean') -> np.array:
    """
    Resamples the values in a 2D array by an integer factor for each axis using a specified stat type

    Args:
        a (np.array): A 2D array of numbers
        f0 (int): An integer resampling factor/ratio evenly divisible into a.shape[0] (along the row axis)
        f1 (int): An integer resampling factor/ratio evenly divisible into a.shape[1] (along the col axis)
        stat (str): mean, max, min, median -> the method for aggregating the numeric values

    Returns:
        resampled np.array, ndim == 2, shape == (a.shape[0] / f0, a.shape[1] / f1)
    """
    assert a.ndim == 2, "valid for 2D arrays only"
    assert a.shape[0] % f0 == 0, "length of axis=0 not evenly divisible by f0"
    assert a.shape[1] % f1 == 0, "length of axis=1 not evenly divisible by f1"
    assert stat in ('mean', 'max', 'min', 'median'), "choose either mean, max, min or median for aggregation stat"

    a0_segments = a.shape[0] / f0
    a1_segments = a.shape[1] / f1
    arr = []
    # split along first axis
    for b in np.split(a, a0_segments, axis=0):
        # split along second axis and aggregate values of each sub array
        if stat == 'mean':
            arr.append([np.nanmean(c) for c in np.split(b, a1_segments, axis=1)])
        elif stat == 'max':
            arr.append([np.nanmax(c) for c in np.split(b, a1_segments, axis=1)])
        elif stat == 'min':
            arr.append([np.nanmin(c) for c in np.split(b, a1_segments, axis=1)])
        elif stat == 'median':
            arr.append([np.nanmedian(c) for c in np.split(b, a1_segments, axis=1)])
    return np.array(arr)


def checkerboard(y: int, x: int) -> np.array:
    """
    Generates an array which alternates 0 and 1 in a checkerboard pattern

    Args:
        y: the height of the array, len(axis=0)
        x: the width of the array, len(axis=1)

    Returns:
        np.array of shape (y, x)
    """
    a = []
    a1 = [i % 2 for i in range(x)]
    a2 = [(i + 1) % 2 for i in range(x)]
    for i in range(y // 2):
        a.append(a1)
        a.append(a2)
    if y % 2 == 1:
        a.append(a1)
    return np.array(a)



# conda install google-api-python-client google-auth-httplib2 google-auth-oauthlib pandas numpy
import numpy as np
import pandas as pd
import requests
from googleapiclient.discovery import build

__all__ = ['read_google_sheet', 'write_google_sheet', 'tmdb_find_movie', 'tmdb_data_for_id']


def read_google_sheet(service: build, sheet_id: str, sheet_range: str,
                      skip_cols: int or list or tuple = None, skip_rows: int or list or tuple = None,
                      columns: bool = True, indexed: bool = False) -> pd.DataFrame:
    """
    Reads a subset of a google sheet via the google sheets api and returns it converted to a pandas DataFrame

    Args:
        service (googleapiclient.discovery.build): a google api service python instance
        sheet_id (str): the identifier for the google sheet
        sheet_range (str): the absolute reference to the cells to be read. For example: "Sheet1!A:H" reads columns
            A -> H on the sheet names Sheet1 within the specified Google Sheet.
        skip_cols (int): an int or iterable of ints of the spreadsheet columns numbers to be ignored in the output
            (e.g. column A = 1, column B = 2, columns A through C = (1, 2, 3)
        skip_rows (int): an int or iterable of ints of the spreadsheet row numbers to be ignored in the output
        indexed (bool): Indicates whether the first column (after skip columns is applied) should be interpreted
            as the index in pandas
        columns (bool): Indicates whether the first row (after skip rows is applied) should be interpreted as
            labels for the data in the columns

    Returns:
        pd.DataFrame
    """
    # Call the Sheets API to get the spreadsheet data
    sheet = service.spreadsheets()
    result = sheet.values().get(spreadsheetId=sheet_id, range=sheet_range).execute()
    values = result.get('values', [])

    # make all the rows the same length
    length = max(map(len, values))
    array = np.array([xi + [''] * (length - len(xi)) for xi in values])

    # delete any rows or columns specified by the user
    if skip_rows is not None:
        array = np.delete(array, np.asarray(skip_rows) - 1, axis=0)  # subtract 1 -> sheets start at 1, np at 0
    if skip_cols is not None:
        array = np.delete(array, np.asarray(skip_cols) - 1, axis=1)

    # extract index and column labels if applicable
    idx = None
    col = None
    if indexed:
        idx = array[:, 0]
        array = np.delete(array, 0, axis=1)
    if columns:
        col = array[0]
        array = np.delete(array, 0, axis=0)
    if indexed and columns:
        idx = np.delete(idx, 0, axis=0)

    return pd.DataFrame(array, columns=col, index=idx)


def write_google_sheet(df: pd.DataFrame, service: build, sheet_id: str, sheet_range: str) -> build:
    """
    writes a pandas dataframe to a google sheet on the range specified

    Args:
        service (googleapiclient.discovery.build): a google api service python instance
        sheet_id (str): the identifier for the google sheet
        sheet_range (str): the absolute reference to the cells to be read. For example: "Sheet1!A:H" reads columns
            A -> H on the sheet names Sheet1 within the specified Google Sheet.
        df (pd.DataFrame): the pandas dataframe to be written to the google sheet

    Returns:

    """
    col = np.asarray(df.columns)
    col = col.reshape(1, len(col))
    body = df.to_numpy()
    body = {'values': np.vstack((col, body)).tolist()}
    return service.spreadsheets().values().update(
        spreadsheetId=sheet_id, range=sheet_range, valueInputOption='RAW', body=body).execute()


def tmdb_find_movie(movie: str, tmdb_api_token: str):
    """
    Search the tmdb api for movies by title

    Args:
        movie (str): the title of a movie
        tmdb_api_token (str): your tmdb v3 api token

    Returns:
        dict
    """
    url = 'https://api.themoviedb.org/3/search/movie?'
    params = {'query': movie, 'language': 'en-US', 'api_key': tmdb_api_token, }
    return requests.get(url, params).json()


def tmdb_data_for_id(tmdb_id: int, tmdb_api_token: str) -> dict:
    """
    Get additional information for a movie for which you already have the ID

    Args:
        tmdb_id (int): the ID for a movie on The Movie Database
        tmdb_api_token (str): your tmdb v3 api token

    Returns:
        dict
    """
    url = f"https://api.themoviedb.org/3/movie/{tmdb_id}?"
    params = {'language': 'en-US', 'api_key': tmdb_api_token, }
    return requests.get(url, params).json()



from setuptools import setup

version = '0.13'

with open("README.md", "r") as readme:
    long_description = readme.read()

with open('requirements.txt', 'r') as req:
    install_requires = req.read().splitlines()

project_urls = dict(Source='https://github.com/rileyhales/pyrch',
                    License='https://choosealicense.com/licenses/bsd-3-clause-clear')

classifiers = [
    'Programming Language :: Python :: 3',
    'Topic :: Scientific/Engineering',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: BSD License',
    'Natural Language :: English',
]

setup(
    name='rch',
    packages=['rch'],
    version=version,
    install_requires=install_requires,
    python_requires='>3',
    description='A package containing the personal python utilities of Riley Hales',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Riley Hales',
    author_email='rchales@byu.edu',
    url='https://rileyhales.com',
    project_urls=project_urls,
    license='BSD 3-Clause Clear License',
    classifiers=classifiers,
)
